#
#   PROBE - New System screen
#

from albow.resource import get_text, get_image, get_font
from albow.vectors import add
from screen import ProbeScreen
from text_writer import TextWriter
from controls import ImageButton

class NewSystemScreen(ProbeScreen):

	background_image = get_image("blank_paper.jpg")
	logo = get_image("nasa_logo.png")
	signature = get_image("signature.png")
	text = get_text("new_system.txt")
	font = get_font(10, "VeraMono.ttf")
	text_color = (0, 0, 0)
	left_margin = 110
	top_margin = 100
	text_width = 50
	logo_pos = (380, 30)
	signature_pos = (250, 489)

	def __init__(self, shell):
		ProbeScreen.__init__(self, shell)
		self.add(ImageButton("MISSION PLANNING", "drawing_board_thumb.png",
			'bottomright', add(self.rect.bottomright, (-20, -20)),
			self.go_to_mission_planning))
	
	def draw(self, surf):
		frame = surf.get_rect()
		bg = self.background_image
		bg_rect = bg.get_rect()
		bg_rect.center = frame.center
		surf.blit(bg, bg_rect.topleft)
		lx, ly = self.logo_pos
		surf.blit(self.logo, (bg_rect.left + lx, bg_rect.top + ly))
		system = self.game.current_system
		budget = "1,000,000"
		text = self.text % {'SYSTEM_NAME': system.name, 'BUDGET': budget}
		writer = TextWriter(surf, self.font, self.text_color,
			bg_rect.left + self.left_margin, bg_rect.top + self.top_margin)
		writer.write_lines(text)
		sx, sy = self.signature_pos
		surf.blit(self.signature, (bg_rect.left + sx, bg_rect.top + sy))
	
	def go_to_mission_planning(self):
		self.shell.go_to_mission_planning()

