#
#   PROBE - Probe design screen
#

from pygame import Rect, draw
from albow.widget import Widget
from albow.controls import Label, Button, TextField
from albow.resource import get_font, get_image
from albow.palette_view import PaletteView
from albow.dialogs import alert
from controls import ImageButton, ChangingImageButton
from screen import ProbeScreen
from vector_algebra import add, subtract
from text_writer import TextWriter
import design, navigation, messages

#debug_bg = (200, 200, 200)
debug_bg = None

class ProbeDesignScreen(ProbeScreen):

	#bg_color = (255, 255, 255) ###
	bg_image = get_image("desk-probe-design.jpg")
	heading_font = get_font(12, "VeraBd.ttf")
	heading_color = (0, 0, 0)
	button_font = get_font(12, "VeraBd.ttf")
	button_color = (102, 204, 255)

	def __init__(self, shell):
		ProbeScreen.__init__(self, shell)
		game = self.game
		# Selectors
		sel_top = 60
		sel_left = 45
		sel_width = 225
		cat_sel = DesignCategorySelector(sel_width, game)
		cat_sel.rect.topleft = (sel_left, sel_top)
		elem_top = cat_sel.rect.bottom + 20
		elem_sel = DesignElementSelector(sel_width, 5, game)
		elem_sel.rect.topleft = (sel_left, elem_top)
		spec_top = elem_sel.rect.bottom + 20
		spec_height = 150
		spec_rect = Rect(sel_left, spec_top, sel_width, spec_height)
		spec_disp = SpecDisplay(spec_rect, game)
		# Design display
		dd_top = 30
		dd_left = elem_sel.rect.right + 110
		dd_width = self.rect.width - 50 - dd_left
		mn_control = MissionNameControl(dd_width, game)
		mn_control.rect.topleft = (dd_left, dd_top)
		bv_top = mn_control.rect.bottom + 40
		bv_display = BasicVehicleDisplay(dd_width, game)
		bv_display.rect.topleft = (dd_left, bv_top)
		in_top = bv_display.rect.bottom + 40
		in_display = InstrumentsDisplay(dd_width, 10, game)
		in_display.rect.topleft = (dd_left, in_top)
		bs_top = in_display.rect.bottom + 40
		bs_display = BoosterStagesDisplay(dd_width, 5, game)
		bs_display.rect.topleft = (dd_left, bs_top)
		dt_top = bs_display.rect.bottom + 20
		dt_display = DesignTotalDisplay(dd_width, game)
		dt_display.rect.topleft = (dd_left, dt_top)
		#dd_topleft = (elem_sel.rect.right + 100, 20)
		#dd_botright = add(self.rect.size, (-20, -20))
		#dd_rect = Rect(dd_topleft, subtract(dd_botright, dd_topleft))
		#des_disp = DesignDisplay(dd_rect, game)
		# Buttons
		but_mid = sel_left + sel_width + 65
		add_button = ChangingImageButton(
			"button-setsquare-80x40.png", border = 2, title = "Add",
			font = self.button_font, fg_color = self.button_color,
			action = self.add_element)
		add_button.rect.midtop = (but_mid, 150)
		remove_button = ChangingImageButton(
			"button-wastebasket-100x110.png", border = 2, title = "Remove",
			font = self.button_font, fg_color = self.button_color,
			action = self.remove_element)
		remove_button.rect.midtop = (but_mid, 300)
		#build_button = Button("Build", action = self.build_vehicle)
		build_button = ImageButton("BUILD", "vab-thumb.jpg",
			action = self.build_vehicle)
		#build_button.rect.topright = add(remove_button.rect.bottomright, (0, 20))
		#build_button.bg_color = (200, 200, 200)
		# Navigation
		mp_button = navigation.mission_planning_button(self)
		mp_button.rect.bottomleft = (20, self.rect.height - 20)
		build_button.rect.bottomleft = add(mp_button.rect.bottomright, (10, 0))
		lp_button = navigation.launch_prep_button(self)
		lp_button.rect.bottomleft = add(build_button.rect.bottomright, (10, 0))
		fq_button = navigation.funding_request_button(self)
		fq_button.rect.bottomleft = add(mp_button.rect.topleft, (0, -10))
		# Adding
		self.add(cat_sel)
		self.add(elem_sel)
		self.add(spec_disp)
		#self.add(des_disp)
		self.add(add_button)
		self.add(remove_button)
		self.add(build_button)
		self.add(mp_button)
		self.add(lp_button)
		self.add(fq_button)
		self.add(mn_control)
		self.add(bv_display)
		self.add(in_display)
		self.add(bs_display)
		self.add(dt_display)
		self.heading_above(cat_sel, "Equipment Categories")
		self.heading_above(elem_sel, "Models")
		self.heading_above(bv_display, "Basic Vehicle")
		self.heading_above(in_display, "Instruments")
		self.heading_above(bs_display, "Booster Stages")

	def heading_above(self, widget, text):
		font = getattr(widget, 'heading_font', None) or self.heading_font
		spacing = getattr(widget, 'heading_spacing', None)
		if spacing is None:
			spacing = 5
		lbl = Label(text, font = font)
		lbl.fg_color = self.heading_color
		lbl.rect.bottomleft = add(widget.rect.topleft, (0, -spacing))
		self.add(lbl)
	
	def draw(self, surf):
		surf.blit(self.bg_image, (0, 0))
	
	def go_to_mission_planning(self):
		self.shell.go_to_mission_planning()

	def go_to_launch_prep(self):
		self.shell.go_to_launch_prep()

	def add_element(self):
		#print "ProbeDesignScreen: Add" ###
		cat = design.categories.selected_item()
		if cat:
			elem = cat.selected_item()
			if elem:
				des = self.game.current_design
				elem.add_to_design(des)
				des.selected_element = elem

	def remove_element(self):
		print "ProbeDesignScreen: Remove" ###
		des = self.game.current_design
		elem = des.selected_element
		if elem:
			elem.remove_from_design(des)
			des.selected_element = None
	
	def build_vehicle(self):
		system = self.game.current_system
		if not system:
			alert("There is no star system to build this vehicle in.")
			return
		if system.vehicle:
			old_vehicle = system.vehicle
			if old_vehicle.state <= old_vehicle.PRELAUNCH:
				message_from_scotty(False,
					"Previous vehicle?",
					"You haven't launched the last vehicle we built you yet!")
				return
		des = self.game.current_design
		if not des.mission_name:
			message_from_scotty(False, "Wazizname?",
				"You might like to give this vehicle a name. Just so we've "
				"got something to call it.")
			return
		try:
			des.check_design(self.game)
		except design.DesignError, e:
			#print "****** Design error:", e ###
			message_from_scotty(False, "Problem with the design", str(e))
			return
		vehicle = des.build_vehicle(self.game)
		system.vehicle = vehicle
		system.objects.append(vehicle)
		message_from_scotty(True,
			"Vehicle construction complete",
			"Vehicle is on the pad. You can prepare it for launch now.")
		self.go_to_launch_prep()


def message_from_scotty(okay, subject, text):
	messages.show_memo(okay,
		"Scott Montgomery, Chief Vehicle Construction Engineer",
		subject,
		text)


class SelectorView(PaletteView):

	bg_color = debug_bg
	heading_font = get_font(10, "VeraBd.ttf")
	heading_spacing = 0
	font = get_font(10, "Vera.ttf")
	fg_color = (0, 0, 0)
	sel_color = (102, 204, 255)
	left_margin = 5

	def __init__(self, width, num_rows, game):
		cell_size = (width + self.left_margin, self.font.get_height() + 5)
		self.game = game
		PaletteView.__init__(self, cell_size, num_rows, ncols = 1)
	
	def draw_item(self, surf, item_no, rect):
		buf = self.font.render(self.item_text(item_no), True, self.fg_color)
		r = buf.get_rect()
		r.midleft = add(rect.midleft, (self.left_margin, 0))
		surf.blit(buf, r)
	
	def draw_prehighlight(self, surf, i, rect):
		#draw.rect(surf, self.sel_color, rect)
		surf.fill(self.sel_color, rect)
	

class DesignCategorySelector(SelectorView):

	def __init__(self, width, game):
		num_rows = design.categories.num_items()
		SelectorView.__init__(self, width, num_rows, game)
	
	def num_items(self):
		return design.categories.num_items()
	
	def click_item(self, item_no, e):
		design.categories.selection = item_no
	
	def item_is_selected(self, item_no):
		return design.categories.selection == item_no
	
	def item_text(self, item_no):
		return design.categories.items[item_no].name


class DesignElementSelector(SelectorView):

	def num_items(self):
		cat = design.categories.selected_item()
		if cat:
			return cat.num_items()
		else:
			return 0
	
	def click_item(self, item_no, e):
		cat = design.categories.selected_item()
		cat.selection = item_no
	
	def item_is_selected(self, item_no):
		cat = design.categories.selected_item()
		return cat.selection == item_no

	def item_text(self, item_no):
		cat = design.categories.selected_item()
		item = cat.items[item_no]
		return "%s %s" % (item.model_name, item.kind)


class SpecDisplay(Widget):

	bg_color = debug_bg
	heading_font = get_font(10, "VeraBd.ttf")
	font = get_font(10, "Vera.ttf")
	fg_color = (0, 0, 0)
	value_margin = 100
	
	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		self.game = game
	
	def draw(self, surf):
		elem = design.selected_element()
		if elem:
			w = TextWriter(surf, font = self.font, color = self.fg_color,
				x = 0, y = 0, tabs = [self.value_margin])
			w.writeln("%s %s" % (elem.model_name, elem.kind), font = self.heading_font)
			w.vspace(5)
			specs = []
			elem.get_specifications(specs)
			frame = surf.get_rect()
			for name, value in specs:
				w.write(name)
				w.tab(1)
				w.writeln(value)


class MissionNameControl(Widget):

	label_font = get_font(12, "VeraBd.ttf")
	font = get_font(12, "Vera.ttf")
	fg_color = (0, 0, 0)

	def __init__(self, width, game):
		rect = Rect(0, 0, width, self.font.get_height() + 3)
		Widget.__init__(self, rect)
		lbl = Label("Mission Name", font = self.label_font)
		lbl.fg_color = self.fg_color
		self.add(lbl)
		fld = MissionNameField(width - lbl.rect.width - 10, game, font = self.font)
		fld.fg_color = self.fg_color
		fld.rect.bottomleft = add(lbl.rect.bottomright, (10, 0))
		self.field = fld
		self.add(fld)
	
	def draw(self, surf):
		fld = self.field
		x1 = fld.rect.left
		x2 = fld.rect.right
		y = fld.rect.bottom + 2
		draw.line(surf, self.fg_color, (x1, y), (x2, y), 1)


class MissionNameField(TextField):

	def __init__(self, width, game, font):
		TextField.__init__(self, width, font)
		self.game = game
	
	def get_text(self):
		des = self.game.current_design
		return des.mission_name
	
	def set_text(self, text):
		des = self.game.current_design
		des.mission_name = text


class DesignDisplaySection(object):

	mass_margin = 300
	cost_margin = 365
	
	def draw_item(self, surf, item_no, rect):
		elem = self.element_no(item_no)
		text = self.element_text(elem)
		self.draw_text(surf, rect, text, 0, 'left')
		self.draw_text(surf, rect, "%s kg" % elem.total_mass(), self.mass_margin, 'right')
		self.draw_text(surf, rect, "$%s" % elem.cost, self.cost_margin, 'right')

	def draw_text(self, surf, rect, text, x, align):
		buf = self.font.render(text, True, self.fg_color)
		r = buf.get_rect()
		#print "DesignDisplaySection.draw_text:", self, repr(text), r ###
		attr = 'mid' + align
		setattr(r, attr, add(rect.midleft, (x, 0)))
		surf.blit(buf, r)


class DesignTotalDisplay(DesignDisplaySection, Widget):

	font = get_font(10, "VeraBd.ttf")
	#font = get_font(10, "Vera.ttf")
	fg_color = (0, 0, 0)

	def __init__(self, width, game):
		rect = Rect(0, 0, width, self.font.get_height())
		Widget.__init__(self, rect)
		self.game = game
	
	def draw(self, surf):
		frame = surf.get_rect()
		des = self.game.current_design
		self.draw_text(surf, frame, "Vehicle Totals", 0, 'left')
		#self.draw_text(surf, frame,
		#	"Total empty mass", self.mass_margin - 50, 'left')
		self.draw_text(surf, frame,
			"%s kg" % des.total_mass(), self.mass_margin, 'right')
		#self.draw_text(surf, frame,
		#	"Total cost (excl fuel)", self.cost_margin - 50, 'left')
		self.draw_text(surf, frame,
			"$%s" % des.total_cost(), self.cost_margin, 'right')


class BasicVehicleDisplay(DesignDisplaySection, SelectorView):

	def __init__(self, width, game):
		SelectorView.__init__(self, width, 1, game)

	def num_items(self):
		des = self.game.current_design
		if des.basic_vehicle:
			return 1
		else:
			return 0
	
	def click_item(self, item_no, e):
		des = self.game.current_design
		des.selected_element = des.basic_vehicle
	
	def item_is_selected(self, item_no):
		des = self.game.current_design
		return des.selected_element is des.basic_vehicle
		
	def element_no(self, item_no):
		des = self.game.current_design
		return des.basic_vehicle
	
	def element_text(self, elem):
		return elem.model_name

	#def item_text(self, item_no):
	#	des = self.game.current_design
	#	elem = des.basic_vehicle
	#	return elem.model_name


class ElementListDisplay(DesignDisplaySection, SelectorView):

	def num_items(self):
		des = self.game.current_design
		return len(self.get_element_list(des))	

	def click_item(self, item_no, e):
		des = self.game.current_design
		des.selected_element = self.get_element_list(des)[item_no]

	def item_is_selected(self, item_no):
		des = self.game.current_design
		return des.selected_element is self.get_element_list(des)[item_no]
	
	def element_no(self, item_no):
		des = self.game.current_design
		return self.get_element_list(des)[item_no]

	def element_text(self, elem):
		return "%s %s" % (elem.model_name, elem.kind)

	#def item_text(self, item_no):
	#	des = self.game.current_design
	#	elem = self.get_element_list(des)[item_no]
	#	return "%s %s" % (elem.model_name, elem.kind)


class InstrumentsDisplay(ElementListDisplay):

	def get_element_list(self, des):
		return des.instruments


class BoosterStagesDisplay(ElementListDisplay):

	def get_element_list(self, des):
		return des.booster_stages


#class DesignDisplay(Widget):
#
#	bg_color = debug_bg
#	heading_font = get_font(12, "VeraBd.ttf")
#	font = get_font(12, "Vera.ttf")
#	fg_color = (0, 0, 0)
#	left_margin = 10
#	top_margin = 10
#
#	def __init__(self, rect, game):
#		Widget.__init__(self, rect)
#		self.game = game
#	
#	def draw(self, surf):
#		des = self.game.current_design
#		w = TextWriter(surf, font = self.font, color = self.fg_color,
#			x = self.left_margin, y = self.top_margin)
#		self.draw_basic_vehicle(w, des.basic_vehicle)
#		w.vspace(20)
#		self.draw_instruments(w, des.instruments)
#		w.vspace(20)
#		self.draw_booster_stages(w, des.booster_stages)
#	
#	def draw_basic_vehicle(self, w, bv):
#		w.writeln("Basic Vehicle", font = self.heading_font)
#		if bv:
#			w.writeln(bv.model_name)
#	
#	def draw_instruments(self, w, insts):
#		w.writeln("Instruments", font = self.heading_font)
#		for inst in insts:
#			w.writeln("%s %s" % (inst.kind, inst.model_name))
#	
#	def draw_booster_stages(self, w, stages):
#		w.writeln("Booster Stages", font = self.heading_font)
#		for stage in stages:
#			w.writeln(stage.model_name)

